/* fishdebug.c -- fishnet debug/print functions */

#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <ctype.h>

#include "fish.h"
#include "fishguts.h"

#ifndef FALSE
#define FALSE 0
#define TRUE !FALSE
#endif

int debuglevel = FISH_DEBUG_ALL;
FILE *debugoutput;

extern void fish_setdebuglevel(const int level) {
  debuglevel = level;
}

extern int fish_getdebuglevel() {
  return debuglevel;
}

extern void fish_setdebugfile(FILE *f) {
  debugoutput = f;
}

extern void fish_debug(const int level, const char* format, ...) {
  char str[10];
  struct timeval timev;
  va_list ap;
  time_t t;


  if (debuglevel >= level) {
    if(debugoutput == NULL) debugoutput = stderr;
    time(&t);
    strftime(str, 10, "%H:%M:%S", localtime(&t));
    fprintf(debugoutput, str);
    gettimeofday(&timev, NULL);
    fprintf(debugoutput, ".%06ld ", (long)timev.tv_usec);
    
    va_start(ap, format);
    vfprintf(debugoutput, format, ap);
    va_end(ap);

    fflush(debugoutput);
  }
}

extern void fish_debugchar(int level, char ch) {
  if (debuglevel >= level) {
    if(debugoutput == NULL) debugoutput = stderr;
    fputc(ch, debugoutput);
    fflush(debugoutput);
  }
}
    
extern void printmessage(const struct fish_message *pfm) {
  unsigned int i;
  static const char *strmessage[] = {
    "Error",
    "Neighborlist",
    "Join Request"
    };

  printf("Message len: %u, type: %s(%u)\n",
         pfm->length, 
         (pfm->type < FISH_MSG_TYPE_MAX) ? strmessage[pfm->type] 
                                         : "Invalid type",
         pfm->type);

  switch(pfm->type) {
  case FISH_MSG_TYPE_ERROR:
    printf("Message: %*s\n", 
           pfm->length - MIN_MESSAGE_SIZE,
           pfm->body.errormessage);
    break;
  case FISH_MSG_TYPE_NEIGHBORLIST:
    for(i = 0; 
        i < (pfm->length - MIN_MESSAGE_SIZE)/sizeof(struct neighbor); 
        i++) {
      const struct neighbor *n = &pfm->body.neighbors[i];
      printf("Neighbor %d: fish address %d, udp %s:%u\n",
             i, 
             n->address, 
             inet_ntoa(n->node_ip_address),
             htons(n->node_udp_port));
    }
    break;
  case FISH_MSG_TYPE_JOINREQUEST:
    printf("fish addr %d, udp port %u\n", 
           pfm->body.joinreq.node_address,
           pfm->body.joinreq.node_udp_port);
    break;
  default:
    printf("invalid message.\n");
    break;
  } /* switch */

}


extern void printneighbors(const struct neighbor *ngh) {
  unsigned int i;
  fish_debug(FISH_DEBUG_TOPOLOGY, "fish: Neighbor list \n");
  for(i = 0; 
        i < MAX_NEIGHBORS; 
        i++) {
      if(ngh[i].address >0)
        fish_debug(FISH_DEBUG_TOPOLOGY,
               "fish:  Neighbor %d: fish address %d, udp %s:%u, loss %f\n",
               i, 
               ngh[i].address, 
               inet_ntoa(ngh[i].node_ip_address),
               htons(ngh[i].node_udp_port),
                   (float) ngh[i].lossrate_x_1000 / (float) 1000);
  }
}

extern void fish_debugframe(int level, const char* msg, 
                            const void *frame, int frame_length) {
  struct packet *pkt = (struct packet *)frame;

//  struct routing_packet *rpkt = (struct routing_packet*)frame;
  int numadv, i;

  struct transport_packet *tpkt = (struct transport_packet*)frame;
  int datalength = frame_length - 
	           (TRANSPORT_HEADER_SIZE + PACKET_HEADER_SIZE);
  int printable;

  /*   char format[100]; */

  switch (pkt->hdr.protocol) {
    case FISH_PROTOCOL_PACKET:
      fish_debug(level, "%s: %s %d -> %d (ttl=%d) \"%s\"\n", 
		 msg, protocolstr[pkt->hdr.protocol],
		 pkt->hdr.src, pkt->hdr.dst, pkt->hdr.ttl,
		 pkt->data);
      break;

    case FISH_PROTOCOL_ECHO:
      {
	struct echo_packet *epkt = (struct echo_packet *)pkt;
	char code_buf[10];
	
	snprintf(code_buf, 10, "%d", epkt->echo_hdr.code);
	code_buf[9]= '\0';

	fish_debug(level, "%s: %s %d -> %d (ttl=%d) ECHO code %s \"%s\"\n",
		   msg, protocolstr[pkt->hdr.protocol],
		   pkt->hdr.src, pkt->hdr.dst, pkt->hdr.ttl,
		   ((epkt->echo_hdr.code == ECHO_PROTOCOL_REQUEST) ? "REQUEST" :
		    (epkt->echo_hdr.code == ECHO_PROTOCOL_RESPONSE) ? "RESPONSE" :
		    code_buf),
		   epkt->data);
      }
      break;
    case FISH_PROTOCOL_ROUTING: 
      numadv = (frame_length - PACKET_HEADER_SIZE) 
               / sizeof(struct route_advertisement);
      fish_debug(level, "%s:\t%s %d -> %d (ttl=%d, %d advertisements)\n",
                 msg, protocolstr[pkt->hdr.protocol],
                 pkt->hdr.src, pkt->hdr.dst, pkt->hdr.ttl,
                 numadv);
#if 0
      for(i=0; i < numadv; i++) {
        fish_debug(level, "\t%d at cost %d\n",
                   rpkt->adv[i].destination_address,
                   rpkt->adv[i].metric);
      }
#endif
      break;
    case FISH_PROTOCOL_TRANSPORT:
      printable = TRUE;
      for (i=0; i < datalength; i++) {
	if (!(isspace(tpkt->data[i]) || isprint(tpkt->data[i]))) {
	  printable = FALSE;
	  break;
        }
      }

      /* 
      snprintf(format, 100, 
               "%%s:\t%%s %%d:%%d -> %%d:%%d (ttl=%%d, seq=%%d, ack=%%d, "
               "length=%%d) %%s%%s %%.%ds\n", datalength);
      */

      fish_debug(level, 
                 "%s:\t%s %d:%d -> %d:%d (ttl=%d, seq=%d, ack=%d, "
                 "length=%d) %s%s %.*s\n",
                 /* format, */
                 msg, protocolstr[pkt->hdr.protocol],
                 tpkt->hdr.src, tpkt->transport_hdr.src_port,
                 tpkt->hdr.dst, tpkt->transport_hdr.dst_port,
                 tpkt->hdr.ttl,
		 tpkt->transport_hdr.seq_no, 
		 tpkt->transport_hdr.ack_no, 
                 datalength,
                 tpkt->transport_hdr.syn ?  "S" : "",
                 tpkt->transport_hdr.fin ? "F" : "",
                 datalength,
                 (printable ? tpkt->data : "binary"));
            
      break;

    default:
      // We're assuming the frame is not a proper packet if it doesn't
      // have a valid protocol field
      fish_debug(level, "%s:\t%s", msg, (char*)frame);
      break;
  }
}
/* vim:set ts=2: */
/*
 * Local Variables:
 * tab-width: 2
 * c-indent-level: 2
 * c-basic-offset: 2
 * End:
 */
